@@ -5,32 +5,43 @@ use strict;
 
 use File::Next 0.40;
 
+use App::Ack::Plugin::Basic;
+
 =head1 NAME
 
 App::Ack - A container for functions for the ack program
 
 =head1 VERSION
 
-Version 1.77_04
+Version 1.89_02
 
 =cut
 
 our $VERSION;
 our $COPYRIGHT;
 BEGIN {
-    $VERSION = '1.77_04';
-    $COPYRIGHT = 'Copyright 2005-2008 Andy Lester, all rights reserved.';
+    $VERSION = '1.89_02';
+    $COPYRIGHT = 'Copyright 2005-2009 Andy Lester, all rights reserved.';
 }
 
+our $fh;
+
+BEGIN {
+    $fh = *STDOUT;
+}
+
+
 our %types;
 our %type_wanted;
 our %mappings;
 our %ignore_dirs;
 
-our $path_sep_regex;
+our $input_from_pipe;
+our $output_to_pipe;
+
+our $dir_sep_chars;
 our $is_cygwin;
 our $is_windows;
-our $to_screen;
 
 use File::Spec ();
 use File::Glob ':glob';
@@ -61,14 +72,16 @@ BEGIN {
 
     %mappings = (
         actionscript => [qw( as mxml )],
-        asm         => [qw( s )],
+        asm         => [qw( asm s )],
+        batch       => [qw( bat cmd )],
         binary      => q{Binary files, as defined by Perl's -B op (default: off)},
         cc          => [qw( c h xs )],
-        cpp         => [qw( cpp cc m hpp hh h )],
+        cfmx        => [qw( cfc cfm cfml )],
+        cpp         => [qw( cpp cc cxx m hpp hh h hxx )],
         csharp      => [qw( cs )],
         css         => [qw( css )],
         elisp       => [qw( el )],
-        erlang      => [qw( erl )],
+        erlang      => [qw( erl hrl )],
         fortran     => [qw( f f77 f90 f95 f03 for ftn fpp )],
         haskell     => [qw( hs lhs )],
         hh          => [qw( h )],
@@ -77,6 +90,7 @@ BEGIN {
         js          => [qw( js )],
         jsp         => [qw( jsp jspx jhtm jhtml )],
         lisp        => [qw( lisp lsp )],
+        lua         => [qw( lua )],
         make        => q{Makefiles},
         mason       => [qw( mas mhtml mpl mtxt )],
         objc        => [qw( m h )],
@@ -87,13 +101,14 @@ BEGIN {
         php         => [qw( php phpt php3 php4 php5 )],
         plone       => [qw( pt cpt metadata cpy py )],
         python      => [qw( py )],
-        ruby        => [qw( rb rhtml rjs rxml )],
-        scheme      => [qw( scm )],
-        shell       => [qw( sh bash csh ksh zsh )],
+        rake        => q{Rakefiles},
+        ruby        => [qw( rb rhtml rjs rxml erb rake )],
+        scheme      => [qw( scm ss )],
+        shell       => [qw( sh bash csh tcsh ksh zsh )],
         skipped     => q{Files, but not directories, normally skipped by ack (default: off)},
         smalltalk   => [qw( st )],
         sql         => [qw( sql ctl )],
-        tcl         => [qw( tcl )],
+        tcl         => [qw( tcl itcl itk )],
         tex         => [qw( tex cls sty )],
         text        => q{Text files, as defined by Perl's -T op (default: off)},
         tt          => [qw( tt tt2 ttml )],
@@ -111,10 +126,13 @@ BEGIN {
         }
     }
 
-    $path_sep_regex = quotemeta( File::Spec->catfile( '', '' ) );
-    $is_cygwin      = ($^O eq 'cygwin');
-    $is_windows     = ($^O =~ /MSWin32/);
-    $to_screen      = -t *STDOUT;
+    # These have to be checked before any filehandle diddling.
+    $output_to_pipe  = not -t *STDOUT;
+    $input_from_pipe = -p STDIN;
+
+    $is_cygwin       = ($^O eq 'cygwin');
+    $is_windows      = ($^O =~ /MSWin32/);
+    $dir_sep_chars   = $is_windows ? quotemeta( '\\/' ) : quotemeta( File::Spec->catfile( '', '' ) );
 }
 
 =head1 SYNOPSIS
@@ -163,7 +181,9 @@ Gets command-line arguments and does the Ack-specific tweaking.
 =cut
 
 sub get_command_line_options {
-    my %opt;
+    my %opt = (
+        pager => $ENV{ACK_PAGER_COLOR} || $ENV{ACK_PAGER},
+    );
 
     my $getopt_specs = {
         1                       => sub { $opt{1} = $opt{m} = 1 },
@@ -173,10 +193,14 @@ sub get_command_line_options {
         'a|all-types'           => \$opt{all},
         'break!'                => \$opt{break},
         c                       => \$opt{count},
-        'color!'                => \$opt{color},
+        'color|colour!'         => \$opt{color},
+        'color-match=s'         => \$ENV{ACK_COLOR_MATCH},
+        'color-filename=s'      => \$ENV{ACK_COLOR_FILENAME},
+        'column!'               => \$opt{column},
         count                   => \$opt{count},
         'env!'                  => sub { }, # ignore this option, it is handled beforehand
         f                       => \$opt{f},
+        flush                   => \$opt{flush},
         'follow!'               => \$opt{follow},
         'g=s'                   => sub { shift; $opt{G} = shift; $opt{f} = 1 },
         'G=s'                   => \$opt{G},
@@ -193,16 +217,19 @@ sub get_command_line_options {
         n                       => \$opt{n},
         o                       => sub { $opt{output} = '$&' },
         'output=s'              => \$opt{output},
+        'pager=s'               => \$opt{pager},
+        'nopager'               => sub { $opt{pager} = undef },
         'passthru'              => \$opt{passthru},
         'print0'                => \$opt{print0},
         'Q|literal'             => \$opt{Q},
+        'smart-case!'           => \$opt{smart_case},
         'sort-files'            => \$opt{sort_files},
         'u|unrestricted'        => \$opt{u},
         'v|invert-match'        => \$opt{v},
         'w|word-regexp'         => \$opt{w},
 
-        'ignore-dirs=s'         => sub { shift; my $dir = remove_path_sep( shift ); $ignore_dirs{$dir} = '--ignore-dirs' },
-        'noignore-dirs=s'       => sub { shift; my $dir = remove_path_sep( shift ); delete $ignore_dirs{$dir} },
+        'ignore-dirs=s'         => sub { shift; my $dir = remove_dir_sep( shift ); $ignore_dirs{$dir} = '--ignore-dirs' },
+        'noignore-dirs=s'       => sub { shift; my $dir = remove_dir_sep( shift ); delete $ignore_dirs{$dir} },
 
         'version'   => sub { print_version_statement(); exit 1; },
         'help|?:s'  => sub { shift; show_help(@_); exit; },
@@ -241,15 +268,25 @@ sub get_command_line_options {
     $parser->getoptions( %{$getopt_specs} ) or
         App::Ack::die( 'See ack --help or ack --man for options.' );
 
+    my $to_screen = not output_to_pipe();
     my %defaults = (
         all            => 0,
-        color          => $to_screen && !$App::Ack::is_windows,
+        color          => $to_screen,
         follow         => 0,
         break          => $to_screen,
         heading        => $to_screen,
         before_context => 0,
         after_context  => 0,
     );
+    if ( $is_windows && $defaults{color} && not $ENV{ACK_PAGER_COLOR} ) {
+        if ( $ENV{ACK_PAGER} || not eval { require Win32::Console::ANSI } ) {
+            $defaults{color} = 0;
+        }
+    }
+    if ( $to_screen && $ENV{ACK_PAGER_COLOR} ) {
+        $defaults{color} = 1;
+    }
+
     while ( my ($key,$value) = each %defaults ) {
         if ( not defined $opt{$key} ) {
             $opt{$key} = $value;
@@ -303,7 +340,7 @@ sub get_command_line_options {
         }
     }
 
-    return %opt;
+    return \%opt;
 }
 
 =head2 def_types_from_ARGV
@@ -328,7 +365,7 @@ sub def_types_from_ARGV {
     ) or App::Ack::die( 'See ack --help or ack --man for options.' );
 
     for my $td (@typedef) {
-        my ($type, $ext) = split '=', $td->[1];
+        my ($type, $ext) = split /=/, $td->[1];
 
         if ( $td->[0] eq 'c' ) {
             # type-set
@@ -351,7 +388,8 @@ sub def_types_from_ARGV {
                 unless exists $mappings{$type};
         }
 
-        my @exts = map { s/^\.//; $_ } split ',', $ext; # %types stores extensions without leading '.'
+        my @exts = split /,/, $ext;
+        s/^\.// for @exts;
 
         if ( !exists $mappings{$type} || ref($mappings{$type}) eq 'ARRAY' ) {
             push @{$mappings{$type}}, @exts;
@@ -399,15 +437,15 @@ sub ignoredir_filter {
     return !exists $ignore_dirs{$_};
 }
 
-=head2 remove_path_sep( $path )
+=head2 remove_dir_sep( $path )
 
 This functions removes a trailing path separator, if there is one, from its argument
 
 =cut
 
-sub remove_path_sep {
+sub remove_dir_sep {
     my $path = shift;
-    $path =~ s/$path_sep_regex$//;
+    $path =~ s/[$dir_sep_chars]$//;
 
     return $path;
 }
@@ -432,10 +470,15 @@ sub filetypes {
 
     return 'skipped' unless is_searchable( $filename );
 
-    return ('make',TEXT) if $filename =~ m{$path_sep_regex?Makefile$}io;
+    my $basename = $filename;
+    $basename =~ s{.*[$dir_sep_chars]}{};
+
+    my $lc_basename = lc $basename;
+    return ('make',TEXT)        if $lc_basename eq 'makefile';
+    return ('rake','ruby',TEXT) if $lc_basename eq 'rakefile';
 
     # If there's an extension, look it up
-    if ( $filename =~ m{\.([^\.$path_sep_regex]+)$}o ) {
+    if ( $filename =~ m{\.([^\.$dir_sep_chars]+)$}o ) {
         my $ref = $types{lc $1};
         return (@{$ref},TEXT) if $ref;
     }
@@ -466,11 +509,11 @@ sub filetypes {
         return;
     }
     my $header = <$fh>;
-    App::Ack::close_file( $fh, $filename ) or return;
+    close $fh;
 
     if ( $header =~ /^#!/ ) {
         return ($1,TEXT)       if $header =~ /\b(ruby|p(?:erl|hp|ython))\b/;
-        return ('shell',TEXT)  if $header =~ /\b(?:ba|c|k|z)?sh\b/;
+        return ('shell',TEXT)  if $header =~ /\b(?:ba|t?c|k|z)?sh\b/;
     }
     else {
         return ('xml',TEXT)    if $header =~ /\Q<?xml /i;
@@ -496,9 +539,9 @@ sub is_searchable {
     my $filename = shift;
 
     # If these are updated, update the --help message
-    return if $filename =~ /\.bak$/;
+    return if $filename =~ /[.]bak$/;
     return if $filename =~ /~$/;
-    return if $filename =~ m{$path_sep_regex?(?:#.+#|core\.\d+|[._].*\.swp)$}o;
+    return if $filename =~ m{[$dir_sep_chars]?(?:#.+#|core\.\d+|[._].*\.swp)$}o;
 
     return 1;
 }
@@ -519,6 +562,11 @@ sub build_regex {
         $str = "$str\\b" if $str =~ /\w$/;
     }
 
+    my $regex_is_lc = $str eq lc $str;
+    if ( $opt->{i} || ($opt->{smart_case} && $regex_is_lc) ) {
+        $str = "(?i)$str";
+    }
+
     return $str;
 }
 
@@ -592,7 +640,7 @@ sub _get_thpppt {
 
 sub _thpppt {
     my $y = _get_thpppt();
-    print "$y ack $_[0]!\n";
+    App::Ack::print( "$y ack $_[0]!\n" );
     exit 0;
 }
 
@@ -616,12 +664,12 @@ sub show_help {
 
     my $ignore_dirs = _listify( sort { _key($a) cmp _key($b) } keys %ignore_dirs );
 
-    print <<"END_OF_HELP";
-Usage: ack [OPTION]... PATTERN [FILES]
+    App::Ack::print( <<"END_OF_HELP" );
+Usage: ack [OPTION]... PATTERN [FILE]
 
 Search for PATTERN in each source file in the tree from cwd on down.
 If [FILES] is specified, then only those files/directories are checked.
-ack may also search STDIN, but only if no FILES are specified, or if
+ack may also search STDIN, but only if no FILE are specified, or if
 one of FILES is "-".
 
 Default switches may be specified in ACK_OPTIONS environment variable or
@@ -632,6 +680,9 @@ Example: ack -i select
 
 Searching:
   -i, --ignore-case     Ignore case distinctions in PATTERN
+  --[no]smart-case      Ignore case distinctions in PATTERN,
+                        only if PATTERN contains no upper case
+                        Ignored if -i is specified
   -v, --invert-match    Invert match: select non-matching lines
   -w, --word-regexp     Force PATTERN to match only whole words
   -Q, --literal         Quote all metacharacters; PATTERN is literal
@@ -653,6 +704,7 @@ Search output:
   -H, --with-filename   Print the filename for each match
   -h, --no-filename     Suppress the prefixing filename on output
   -c, --count           Show number of lines matching per file
+  --column              Show the column number of the first match
 
   -A NUM, --after-context=NUM
                         Print NUM lines of trailing context after matching
@@ -667,6 +719,10 @@ Search output:
                         only works with -f, -g, -l, -L or -c.
 
 File presentation:
+  --pager=COMMAND       Pipes all ack output through COMMAND.
+                        Ignored if output is redirected.
+  --nopager             Do not send output through a pager.  Cancels any
+                        setting in ~/.ackrc, ACK_PAGER or ACK_PAGER_COLOR.
   --[no]heading         Print a filename heading above each file's results.
                         (default: on when used interactively)
   --[no]break           Print a break between results from different files.
@@ -675,6 +731,12 @@ File presentation:
   --nogroup             Same as --noheading --nobreak
   --[no]color           Highlight the matching text (default: on unless
                         output is redirected, or on Windows)
+  --[no]colour          Same as --[no]color
+  --color-filename=COLOR
+  --color-match=COLOR   Set the color for matches and filenames.
+  --flush               Flush output immediately, even when ack is used
+                        non-interactively (when output goes to a pipe or
+                        file).
 
 File finding:
   -f                    Only print the files found, without searching.
@@ -696,13 +758,13 @@ File inclusion/exclusion:
   --type=noperl         Exclude Perl files.
                         See "ack --help type" for supported filetypes.
 
-  --type-add TYPE=.EXTENSION[,.EXT2[,...]]
-                        Files with the given EXTENSION(s) are recognized as
-                        being of (the existing) type TYPE
   --type-set TYPE=.EXTENSION[,.EXT2[,...]]
                         Files with the given EXTENSION(s) are recognized as
                         being of type TYPE. This replaces an existing
                         definition for type TYPE.
+  --type-add TYPE=.EXTENSION[,.EXT2[,...]]
+                        Files with the given EXTENSION(s) are recognized as
+                        being of (the existing) type TYPE
 
   --[no]follow          Follow symlinks.  Default is off.
 
@@ -716,14 +778,19 @@ File inclusion/exclusion:
     /core\\.\\d+\$/   - core dumps
 
 Miscellaneous:
+  --noenv               Ignore environment variables and ~/.ackrc
   --help                This help
   --man                 Man page
   --version             Display version & copyright
   --thpppt              Bill the Cat
+
+Exit status is 0 if match, 1 if no match.
+
+This is version $VERSION of ack.
 END_OF_HELP
 
     return;
-}
+ }
 
 
 =head2 show_help_types()
@@ -733,7 +800,7 @@ Display the filetypes help subpage.
 =cut
 
 sub show_help_types {
-    print <<'END_OF_HELP';
+    App::Ack::print( <<'END_OF_HELP' );
 Usage: ack [OPTION]... PATTERN [FILES]
 
 The following is the list of filetypes supported by ack.  You can
@@ -757,7 +824,7 @@ END_OF_HELP
         if ( ref $ext_list ) {
             $ext_list = join( ' ', map { ".$_" } @{$ext_list} );
         }
-        printf( "    --[no]%-*.*s %s\n", $maxlen, $maxlen, $type, $ext_list );
+        App::Ack::print( sprintf( "    --[no]%-*.*s %s\n", $maxlen, $maxlen, $type, $ext_list ) );
     }
 
     return;
@@ -777,32 +844,42 @@ sub _listify {
     return Text::Wrap::wrap( '', '    ', $str );
 }
 
-=head2 get_version_statement( $copyright )
+=head2 get_version_statement
 
 Returns the version information for ack.
 
 =cut
 
 sub get_version_statement {
+    require Config;
+
     my $copyright = get_copyright();
+    my $this_perl = $Config::Config{perlpath};
+    if ($^O ne 'VMS') {
+        my $ext = $Config::Config{_exe};
+        $this_perl .= $ext unless $this_perl =~ m/$ext$/i;
+    }
+
     return <<"END_OF_VERSION";
 ack $VERSION
+Running under Perl $] at $this_perl
 
 $copyright
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This program is free software; you can redistribute it and/or modify
+it under the terms of either: the GNU General Public License as
+published by the Free Software Foundation; or the Artistic License.
 END_OF_VERSION
 }
 
-=head2 print_version_statement( $copyright )
+=head2 print_version_statement
 
 Prints the version information for ack.
 
 =cut
 
 sub print_version_statement {
-    print get_version_statement();
+    App::Ack::print( get_version_statement() );
 
     return;
 }
@@ -819,17 +896,15 @@ sub get_copyright {
 
 =head2 load_colors
 
-Set default colors, load Term::ANSIColor on non Windows platforms
+Set default colors, load Term::ANSIColor
 
 =cut
 
 sub load_colors {
-    if ( not $is_windows ) {
-        eval 'use Term::ANSIColor ()';
+    eval 'use Term::ANSIColor ()';
 
-        $ENV{ACK_COLOR_MATCH}    ||= 'black on_yellow';
-        $ENV{ACK_COLOR_FILENAME} ||= 'bold green';
-    }
+    $ENV{ACK_COLOR_MATCH}    ||= 'black on_yellow';
+    $ENV{ACK_COLOR_FILENAME} ||= 'bold green';
 
     return;
 }
@@ -860,85 +935,41 @@ sub is_interesting {
 }
 
 
-=head2 open_file( $filename )
 
-Opens the file specified by I<$filename> and returns a filehandle and
-a flag that says whether it could be binary.
-
-If there's a failure, it throws a warning and returns an empty list.
-
-=cut
-
-sub open_file {
+# print subs added in order to make it easy for a third party
+# module (such as App::Wack) to redefine the display methods
+# and show the results in a different way.
+sub print                   { print {$fh} @_ }
+sub print_first_filename    { App::Ack::print( $_[0], "\n" ) }
+sub print_blank_line        { App::Ack::print( "\n" ) }
+sub print_separator         { App::Ack::print( "--\n" ) }
+sub print_filename          { App::Ack::print( $_[0], $_[1] ) }
+sub print_line_no           { App::Ack::print( $_[0], $_[1] ) }
+sub print_column_no         { App::Ack::print( $_[0], $_[1] ) }
+sub print_count {
     my $filename = shift;
+    my $nmatches = shift;
+    my $ors = shift;
+    my $count = shift;
 
-    my $fh;
-    my $could_be_binary;
-
-    if ( $filename eq '-' ) {
-        $fh = *STDIN;
-        $could_be_binary = 0;
-    }
-    else {
-        if ( !open( $fh, '<', $filename ) ) {
-            App::Ack::warn( "$filename: $!" );
-            return;
-        }
-        $could_be_binary = 1;
-    }
-
-    return ($fh,$could_be_binary);
+    App::Ack::print( $filename );
+    App::Ack::print( ':', $nmatches ) if $count;
+    App::Ack::print( $ors );
 }
 
-=head2 close_file( $fh, $filename )
+sub print_count0 {
+    my $filename = shift;
+    my $ors = shift;
 
-Close L<$fh> opened from L<$filename>.
-
-=cut
-
-sub close_file {
-    if ( close $_[0] ) {
-        return 1;
-    }
-    App::Ack::warn( "$_[1]: $!" );
-    return 0;
+    App::Ack::print( $filename, ':0', $ors );
 }
 
 
-=head2 needs_line_scan( $fh, $regex, \%opts )
+=head2 search_resource( $resource, \%opt )
 
-Slurp up an entire file up to 100K, see if there are any matches
-in it, and if so, let us know so we can iterate over it directly.
-If it's bigger than 100K or the match is inverted, we have to do
-the line-by-line, too.
+Main search method.
 
-=cut
-
-sub needs_line_scan {
-    my $fh = shift;
-    my $regex = shift;
-    my $opt = shift;
-
-    return 1 if $opt->{v};
-
-    my $size = -s $fh;
-
-    if ( $size > 100_000 ) {
-        return 1;
-    }
-
-    my $buffer;
-    my $rc = sysread( $fh, $buffer, $size );
-    return 0 unless $rc && ( $rc == $size );
-
-    $regex = $opt->{i} ? qr/$regex/im : qr/$regex/m;
-    return ( $buffer =~ /$regex/ );
-}
-
-
-=head2 search( $fh, $could_be_binary, $filename, \%opt )
-
-Main search method
+Assumes an open resource, and that the caller will close the resource.
 
 =cut
 
@@ -949,16 +980,16 @@ Main search method
 
     my $keep_context;
 
-    my $last_output_line; # number of the last line that has been output
-    my $any_output;       # has there been any output for the current file yet
+    my $last_output_line;             # number of the last line that has been output
+    my $any_output;                   # has there been any output for the current file yet
     my $context_overall_output_count; # has there been any output at all
 
-sub search {
-    my $fh = shift;
-    my $could_be_binary = shift;
-    $filename = shift;
+sub search_resource {
+    my $res = shift;
     my $opt = shift;
 
+    $filename = $res->name();
+
     my $v = $opt->{v};
     my $passthru = $opt->{passthru};
     my $max = $opt->{m};
@@ -975,10 +1006,9 @@ sub search {
         undef $regex; # Don't match when printing matching line
     }
     else {
-        $regex = $opt->{i} ? qr/$opt->{regex}/i : qr/$opt->{regex}/;
+        $regex = qr/$opt->{regex}/;
     }
 
-
     # for context processing
     $last_output_line = -1;
     $any_output = 0;
@@ -991,13 +1021,14 @@ sub search {
     my $before_starts_at_line;
     my $after = 0; # number of lines still to print after a match
 
-    while (<$fh>) {
+    while ( $res->next_text ) {
         # XXX Optimize away the case when there are no more @lines to find.
+        # XXX $has_lines, $passthru and $v never change.  Optimize.
         if ( $has_lines
                ? $. != $lines[0]  # $lines[0] should be a scalar
-               : $v ? /$regex/o : !/$regex/o ) {
+               : $v ? m/$regex/ : !m/$regex/ ) {
             if ( $passthru ) {
-                print;
+                App::Ack::print( $_ );
                 next;
             }
 
@@ -1027,17 +1058,14 @@ sub search {
 
         # print an empty line as a divider before first line in each file (not before the first file)
         if ( !$any_output && $opt->{show_filename} && $opt->{break} && defined( $context_overall_output_count ) ) {
-            print "\n";
+            App::Ack::print_blank_line();
         }
 
         shift @lines if $has_lines;
 
-        if ( $could_be_binary ) {
-            if ( -B $filename ) {
-                print "Binary file $filename matches\n";
-                last;
-            }
-            $could_be_binary = 0;
+        if ( $res->is_binary ) {
+            App::Ack::print( "Binary file $filename matches\n" );
+            last;
         }
         if ( $keep_context ) {
             if ( @before ) {
@@ -1058,15 +1086,9 @@ sub search {
     } # while
 
     return $nmatches;
-}   # search()
+}   # search_resource()
 
 
-sub _print_first_filename { print $_[0], "\n"; }
-sub _print_separator      { print "--\n"; }
-sub _print                { print "$_[0]"; }
-sub _print_filename       { print $_[0], $_[1]; }
-sub _print_line_no        { print $_[0], $_[1]; }
-
 =head2 print_match_or_context( $opt, $is_match, $starting_line_no, @lines )
 
 Prints out a matching line or a line of context around a match.
@@ -1078,9 +1100,10 @@ sub print_match_or_context {
     my $is_match = shift; # is there a match on the line?
     my $line_no  = shift;
 
-    my $color = $opt->{color};
-    my $heading = $opt->{heading};
+    my $color         = $opt->{color};
+    my $heading       = $opt->{heading};
     my $show_filename = $opt->{show_filename};
+    my $show_column   = $opt->{column};
 
     if ( $show_filename ) {
         if ( not defined $display_filename ) {
@@ -1089,7 +1112,7 @@ sub print_match_or_context {
                     ? Term::ANSIColor::colored( $filename, $ENV{ACK_COLOR_FILENAME} )
                     : $filename;
             if ( $heading && !$any_output ) {
-                _print_first_filename($display_filename);
+                App::Ack::print_first_filename($display_filename);
             }
         }
     }
@@ -1100,7 +1123,7 @@ sub print_match_or_context {
         if ( $keep_context && !$output_func ) {
             if ( ( $last_output_line != $line_no - 1 ) &&
                 ( $any_output || ( !$heading && defined( $context_overall_output_count ) ) ) ) {
-                _print_separator();
+                App::Ack::print_separator();
             }
             # to ensure separators between different files when --noheading
 
@@ -1108,16 +1131,18 @@ sub print_match_or_context {
         }
 
         if ( $show_filename ) {
-            _print_filename($display_filename, $sep) if not $heading;
-            _print_line_no($line_no, $sep);
+            App::Ack::print_filename($display_filename, $sep) if not $heading;
+            App::Ack::print_line_no($line_no, $sep);
         }
 
         if ( $output_func ) {
             while ( /$regex/go ) {
-                _print( $output_func->() . "\n" );
+                App::Ack::print( $output_func->() . "\n" );
             }
         }
         else {
+            my $col = $-[0] + 1;
+
             if ( $color && $is_match && $regex &&
                  s/$regex/Term::ANSIColor::colored( substr($_, $-[0], $+[0] - $-[0]), $ENV{ACK_COLOR_MATCH} )/eg ) {
                 # At the end of the line reset the color and remove newline
@@ -1127,7 +1152,10 @@ sub print_match_or_context {
                 # remove any kind of newline at the end of the line
                 s/[\r\n]*\z//;
             }
-            _print($_ . "\n");
+            if ( $show_column ) {
+                App::Ack::print_column_no( $col, $sep );
+            }
+            App::Ack::print($_ . "\n");
         }
         $any_output = 1;
         ++$context_overall_output_count;
@@ -1137,7 +1165,7 @@ sub print_match_or_context {
     return;
 } # print_match_or_context()
 
-} # scope around search() and print_match_or_context()
+} # scope around search_resource() and print_match_or_context()
 
 
 =head2 search_and_list( $fh, $filename, \%opt )
@@ -1148,19 +1176,18 @@ show lines.
 =cut
 
 sub search_and_list {
-    my $fh = shift;
-    my $filename = shift;
+    my $res = shift;
     my $opt = shift;
 
     my $nmatches = 0;
     my $count = $opt->{count};
     my $ors = $opt->{print0} ? "\0" : "\n"; # output record separator
 
-    my $regex = $opt->{i} ? qr/$opt->{regex}/i : qr/$opt->{regex}/;
+    my $regex = qr/$opt->{regex}/;
 
     if ( $opt->{v} ) {
-        while (<$fh>) {
-            if ( /$regex/o ) {
+        while ( $res->next_text ) {
+            if ( /$regex/ ) {
                 return 0 unless $count;
             }
             else {
@@ -1169,8 +1196,8 @@ sub search_and_list {
         }
     }
     else {
-        while (<$fh>) {
-            if ( /$regex/o ) {
+        while ( $res->next_text ) {
+            if ( /$regex/ ) {
                 ++$nmatches;
                 last unless $count;
             }
@@ -1178,12 +1205,10 @@ sub search_and_list {
     }
 
     if ( $nmatches ) {
-        print $filename;
-        print ':', $nmatches if $count;
-        print $ors;
+        App::Ack::print_count( $res->name, $nmatches, $ors, $count );
     }
     elsif ( $count && !$opt->{l} ) {
-        print "$filename:0", $ors;
+        App::Ack::print_count0( $res->name, $ors );
     }
 
     return $nmatches ? 1 : 0;
@@ -1218,7 +1243,7 @@ sub print_files {
     my $ors = $opt->{print0} ? "\0" : "\n";
 
     while ( defined ( my $file = $iter->() ) ) {
-        print $file, $ors;
+        App::Ack::print $file, $ors;
         last if $opt->{1};
     }
 
@@ -1237,14 +1262,17 @@ sub print_files_with_matches {
 
     my $nmatches = 0;
     while ( defined ( my $filename = $iter->() ) ) {
-        my ($fh) = App::Ack::open_file( $filename );
-        next unless defined $fh; # error while opening file
-        $nmatches += App::Ack::search_and_list( $fh, $filename, $opt );
-        App::Ack::close_file( $fh, $filename );
-        last if $nmatches && $opt->{1};
+        my $repo = App::Ack::Repository::Basic->new( $filename );
+        my $res;
+        while ( $res = $repo->next_resource() ) {
+            $nmatches += search_and_list( $res, $opt );
+            $res->close();
+            last if $nmatches && $opt->{1};
+        }
+        $repo->close();
     }
 
-    return;
+    return $nmatches;
 }
 
 =head2 print_matches( $iter, $opt )
@@ -1262,25 +1290,38 @@ sub print_matches {
 
     my $nmatches = 0;
     while ( defined ( my $filename = $iter->() ) ) {
-        my ($fh,$could_be_binary) = App::Ack::open_file( $filename );
-        next unless defined $fh; # error while opening file
-        my $needs_line_scan;
-        if ( $opt->{regex} && !$opt->{passthru} ) {
-            $needs_line_scan = App::Ack::needs_line_scan( $fh, $opt->{regex}, $opt );
-            if ( $needs_line_scan ) {
-                seek( $fh, 0, 0 );
-            }
+        my $repo;
+        my $tarballs_work = 0;
+        if ( $tarballs_work && $filename =~ /\.tar\.gz$/ ) {
+            App::Ack::die( 'Not working here yet' );
+            require App::Ack::Repository::Tar; # XXX Error checking
+            $repo = App::Ack::Repository::Tar->new( $filename );
         }
         else {
-            $needs_line_scan = 1;
+            $repo = App::Ack::Repository::Basic->new( $filename );
         }
-        if ( $needs_line_scan ) {
-            $nmatches += App::Ack::search( $fh, $could_be_binary, $filename, $opt );
+        $repo or next;
+
+        while ( my $res = $repo->next_resource() ) {
+            my $needs_line_scan;
+            if ( $opt->{regex} && !$opt->{passthru} ) {
+                $needs_line_scan = $res->needs_line_scan( $opt );
+                if ( $needs_line_scan ) {
+                    $res->reset();
+                }
+            }
+            else {
+                $needs_line_scan = 1;
+            }
+            if ( $needs_line_scan ) {
+                $nmatches += search_resource( $res, $opt );
+            }
+            $res->close();
         }
-        App::Ack::close_file( $fh, $filename );
         last if $nmatches && $opt->{1};
+        $repo->close();
     }
-    return;
+    return  $nmatches;
 }
 
 =head2 filetype_setup()
@@ -1290,16 +1331,61 @@ Minor housekeeping before we go matching files.
 =cut
 
 sub filetype_setup {
-    my $filetypes_supported_set = App::Ack::filetypes_supported_set();
+    my $filetypes_supported_set = filetypes_supported_set();
     # If anyone says --no-whatever, we assume all other types must be on.
     if ( !$filetypes_supported_set ) {
-        for my $i ( keys %App::Ack::type_wanted ) {
-            $App::Ack::type_wanted{$i} = 1 unless ( defined( $App::Ack::type_wanted{$i} ) || $i eq 'binary' || $i eq 'text' || $i eq 'skipped' );
+        for my $i ( keys %type_wanted ) {
+            $type_wanted{$i} = 1 unless ( defined( $type_wanted{$i} ) || $i eq 'binary' || $i eq 'text' || $i eq 'skipped' );
         }
     }
     return;
 }
 
+=head2 expand_filenames( \@ARGV )
+
+Returns reference to list of expanded filename globs (Win32 only).
+
+=cut
+
+EXPAND_FILENAMES_SCOPE: {
+    my $filter;
+
+    sub expand_filenames {
+        my $argv = shift;
+
+        my $attr;
+        my @files;
+
+        foreach my $pattern ( @{$argv} ) {
+            my @results = bsd_glob( $pattern );
+
+            if (@results == 0) {
+                @results = $pattern; # Glob didn't match, pass it thru unchanged
+            }
+            elsif ( (@results > 1) or ($results[0] ne $pattern) ) {
+                if (not defined $filter) {
+                    eval 'require Win32::File;';
+                    if ($@) {
+                        $filter = 0;
+                    }
+                    else {
+                        $filter = Win32::File::HIDDEN()|Win32::File::SYSTEM();
+                    }
+                } # end unless we've tried to load Win32::File
+                if ( $filter ) {
+                    # Filter out hidden and system files:
+                    @results = grep { not(Win32::File::GetAttributes($_, $attr) and $attr & $filter) } @results;
+                    App::Ack::warn( "$pattern: Matched only hidden files" ) unless @results;
+                } # end if we can filter by file attributes
+            } # end elsif this pattern got expanded
+
+            push @files, @results;
+        } # end foreach pattern
+
+        return \@files;
+    } # end expand_filenames
+} # EXPAND_FILENAMES_SCOPE
+
 
 =head2 get_starting_points( \@ARGV, \%opt )
 
@@ -1314,7 +1400,8 @@ sub get_starting_points {
     my @what;
 
     if ( @{$argv} ) {
-        @what = $App::Ack::is_windows ? <@{$argv}> : @{$argv};
+        @what = @{ $is_windows ? expand_filenames($argv) : $argv };
+        $_ = File::Next::reslash( $_ ) for @what;
 
         # Show filenames unless we've specified one single file
         $opt->{show_filename} = (@what > 1) || (!-f $what[0]);
@@ -1324,10 +1411,8 @@ sub get_starting_points {
         $opt->{show_filename} = 1;
     }
 
-    # Barf if the starting points don't exist
     for my $start_point (@what) {
-        App::Ack::warn("$start_point: No such file or directory") unless -e $start_point;
-        $start_point =~ s{/}{\\}g if $App::Ack::is_windows;
+        App::Ack::warn( "$start_point: No such file or directory" ) unless -e $start_point;
     }
     return \@what;
 }
@@ -1341,31 +1426,99 @@ Return the File::Next file iterator
 
 sub get_iterator {
     my $what = shift;
-    my $opt = shift;
+    my $opt  = shift;
 
-    # Starting points are always search, no matter what
-    my $is_starting_point = sub { return grep { $_ eq $_[0] } @$what };
+    # Starting points are always searched, no matter what
+    my %starting_point = map { ($_ => 1) } @{$what};
+
+    my $g_regex = defined $opt->{G} ? qr/$opt->{G}/ : undef;
+    my $file_filter;
+
+    if ( $g_regex ) {
+        $file_filter
+            = $opt->{u}   ? sub { $File::Next::name =~ /$g_regex/ } # XXX Maybe this should be a 1, no?
+            : $opt->{all} ? sub { $starting_point{ $File::Next::name } || ( $File::Next::name =~ /$g_regex/ && is_searchable( $File::Next::name ) ) }
+            :               sub { $starting_point{ $File::Next::name } || ( $File::Next::name =~ /$g_regex/ && is_interesting( @_ ) ) }
+            ;
+    }
+    else {
+        $file_filter
+            = $opt->{u}   ? sub {1}
+            : $opt->{all} ? sub { $starting_point{ $File::Next::name } || is_searchable( $File::Next::name ) }
+            :               sub { $starting_point{ $File::Next::name } || is_interesting( @_ ) }
+            ;
+    }
+
+    my $descend_filter
+        = $opt->{n} ? sub {0}
+        : $opt->{u} ? sub {1}
+        : \&ignoredir_filter;
 
-    my $file_filter = $opt->{u}   && defined $opt->{G} ? sub { $File::Next::name =~ /$opt->{G}/o }
-                    : $opt->{all} && defined $opt->{G} ? sub { $is_starting_point->( $File::Next::name ) || ( $File::Next::name =~ /$opt->{G}/o && App::Ack::is_searchable( $File::Next::name ) ) }
-                    : $opt->{u}                        ? sub {1}
-                    : $opt->{all}                      ? sub { $is_starting_point->( $File::Next::name ) || App::Ack::is_searchable( $File::Next::name ) }
-                    : defined $opt->{G}                ? sub { $is_starting_point->( $File::Next::name ) || ( $File::Next::name =~ /$opt->{G}/o && App::Ack::is_interesting( @_ ) ) }
-                    :                                    sub { $is_starting_point->( $File::Next::name ) || App::Ack::is_interesting( @_ ) }
-                    ;
     my $iter =
         File::Next::files( {
             file_filter     => $file_filter,
-            descend_filter  => $opt->{n}
-                                    ? sub {0}
-                                    : $opt->{u}
-                                        ? sub {1}
-                                        : \&App::Ack::ignoredir_filter,
+            descend_filter  => $descend_filter,
             error_handler   => sub { my $msg = shift; App::Ack::warn( $msg ) },
             sort_files      => $opt->{sort_files},
             follow_symlinks => $opt->{follow},
-        }, @$what );
+        }, @{$what} );
     return $iter;
 }
 
+
+sub set_up_pager {
+    my $command = shift;
+
+    return unless App::Ack::output_to_pipe();
+
+    my $pager;
+    if ( not open( $pager, '|-', $command ) ) {
+        App::Ack::die( qq{Unable to pipe to pager "$command": $!} );
+    }
+    $fh = $pager;
+
+    return;
+}
+
+=head2 input_from_pipe()
+
+Returns true if ack's input is coming from a pipe.
+
+=cut
+
+sub input_from_pipe {
+    return $input_from_pipe;
+}
+
+
+=head2 output_to_pipe()
+
+Returns true if ack's input is coming from a pipe.
+
+=cut
+
+sub output_to_pipe {
+    return $output_to_pipe;
+}
+
+
+=head1 COPYRIGHT & LICENSE
+
+Copyright 2005-2009 Andy Lester, all rights reserved.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of either:
+
+=over 4
+
+=item * the GNU General Public License as published by the Free
+Software Foundation; either version 1, or (at your option) any later
+version, or
+
+=item * the "Artistic License" which comes with Perl 5.
+
+=back
+
+=cut
+
 1; # End of App::Ack
