@@ -2,6 +2,10 @@
 use File::Next ();
 use App::Ack ();
 
+sub prep_environment {
+    delete @ENV{qw( ACK_OPTIONS ACKRC ACK_PAGER )};
+}
+
 # capture stderr output into this file
 my $catcherr_file = 'stderr.log';
 
@@ -9,7 +13,7 @@ sub is_win32 {
     return $^O =~ /Win32/;
 }
 
-# capture-stderr is executing ack-standalone and storing the stderr output in
+# capture-stderr is executing ack and storing the stderr output in
 # $catcherr_file in a portable way.
 #
 # The quoting of command line arguments depends on the OS
@@ -17,13 +21,17 @@ sub build_command_line {
     my @args = @_;
 
     if ( is_win32() ) {
-        for ( @args ) { s/"/\\"/g; $_ = qq("$_"); }
+        for ( @args ) {
+            s/(\\+)$/$1$1/;     # Double all trailing backslashes
+            s/"/\\"/g;          # Backslash all quotes
+            $_ = qq{"$_"};
+        }
     }
     else {
         @args = map { quotemeta $_ } @args;
     }
 
-    return "$^X -T ./capture-stderr $catcherr_file ./ack-standalone @args";
+    return "$^X -T ./capture-stderr $catcherr_file ./ack @args";
 }
 
 sub slurp {
@@ -50,23 +58,36 @@ sub run_ack {
             or diag( join( "\n", "STDERR:", @{$stderr} ) );
     }
 
-    return @{$stdout};
+    return wantarray ? @{$stdout} : join( "\n", @{$stdout} );
 }
 
+{ # scope for $ack_return_code;
+
+# capture returncode
+our $ack_return_code;
+
 sub run_ack_with_stderr {
     my @args = @_;
 
     my @stdout;
     my @stderr;
 
-    my $cmd = build_command_line( @args );
-    @stdout = `$cmd`;
+    if ( !grep { $_ =~ /^--(no)?env$/ } @args ) {
+        unshift( @args, '--noenv' );
+    }
 
-    open( CATCHERR, '<', $catcherr_file );
-    while( <CATCHERR> ) {
+    my $cmd = build_command_line( @args );
+
+    @stdout = `$cmd`;
+    my ($sig,$core,$rc) = (($? & 127),  ($? & 128) , ($? >> 8));
+    $ack_return_code = $rc;
+    ## XXX what do do with $core or $sig?
+
+    open( my $fh, '<', $catcherr_file ) or die $!;
+    while ( <$fh> ) {
         push( @stderr, $_ );
     }
-    close CATCHERR;
+    close $fh or die $!;
     unlink $catcherr_file;
 
     chomp @stdout;
@@ -74,6 +95,12 @@ sub run_ack_with_stderr {
     return ( \@stdout, \@stderr );
 }
 
+sub get_rc {
+    return $ack_return_code;
+}
+
+} # scope for $ack_return_code
+
 sub pipe_into_ack {
     my $input = shift;
     my @args = @_;
